{ ******************************************************************
  Contour plot of a two-dimensional function
  ****************************************************************** }

program contour;

uses
  utypes, uplot;

{ ******************************************************************
  Define here the function to be plotted
  ****************************************************************** }

const
  FuncName = 'Sin(Sqrt(X^2 + Y^2)) + 0.5 / Sqrt((X + 3.05)^2 + Y^2)';

const
  Xmin = - TwoPi; Xmax = TwoPi; Xstep = Pi;  { Ox scale }
  Ymin = - TwoPi; Ymax = TwoPi; Ystep = Pi;  { Oy scale }

const
  NpX = 60; NpY = 60;  { Number of grid points }

const
  Nc = 30;  { Number of contours }

function Func(X, Y : Float) : Float;

const
  C = 3.05;

var
  X2, Y2, Xc, Xc2 : Float;

begin
  X2 := X * X;
  Y2 := Y * Y;
  Xc := X + C;
  Xc2 := Xc * Xc;

  Func := Sin(Sqrt(X2 + Y2)) + 0.5 / Sqrt(Xc2 + Y2)
end;

{ ******************************************************************
  Main program
  ****************************************************************** }

var
  Dx, Dy : Float;                           { Increments of X and Y }
  Fmin   : Float;                           { Min. function value }
  Fmax   : Float;                           { Max. function value }
  H      : Float;                           { Increment for levels }
  X      : array[0..NpX] of Float;          { X coordinates }
  Y      : array[0..NpY] of Float;          { Y coordinates }
  Z      : array[0..(Nc - 1)] of Float;     { Contour array }
  F      : array[0..NpX, 0..NpY] of Float;  { Function values }
  I, J   : Integer;                         { Loop variables }

begin
  { Initialize graphics }

  if not InitGraphics(9, 2, '../../chr') then
    begin
      Writeln('Unable to set graphic mode');
      Exit;
    end
  else
    Writeln('OK');

  { Set graphic area so that it will look approximately square }

  SetWindow(24, 76, 15, 85, True);

  { Set scales and plot axes }

  SetOxScale(LinScale, Xmin, Xmax, Xstep);
  SetOyScale(LinScale, Ymin, Ymax, Ystep);

  PlotOxAxis;
  PlotOyAxis;

  PlotGrid(BothGrid);

  SetGraphTitle(FuncName);
  WriteGraphTitle;

  { Generate grid points }

  Dx := (Xmax - Xmin) / NpX;
  Dy := (Ymax - Ymin) / NpY;

  X[0] := Xmin;
  for I := 1 to NpX do
    X[I] := X[I - 1] + Dx;

  Y[0] := Ymin;
  for J := 1 to NpY do
    Y[J] := Y[J - 1] + Dy;

  { Compute function values }

  Fmin := Func(Xmin, Ymin);
  Fmax := Fmin;

  for I := 0 to NpX do
    for J := 0 to NpY do
      begin
        F[I,J] := Func(X[I], Y[J]);
        if F[I,J] < Fmin then
          Fmin := F[I,J]
        else if F[I,J] > Fmax then
          Fmax := F[I,J];
      end;

  { Define levels }

  H := (Fmax - Fmin) / (Nc + 1);

  for I := 0 to Nc - 1 do
    Z[I] := Fmin + (I + 1) * H;

  { Plot contour }

  ConRec(X, Y, Z, F);

  ReadLn;

  LeaveGraphics;
end.
