{ **********************************************************************
  This program computes the execution times of the mathematical
  functions provided in GPC and GPMath. The results are displayed 
  and written in the file SPEED.OUT

  NB: The execution time of each function takes into account the
      computation of random arguments. This corresponds to the
      execution time of the function Random.
  ********************************************************************** }

program speed;

uses
  GPC, utypes, umath, utrigo, uhyper, ugamma, uigamma, ubeta, uibeta;

const
  NFUNC = 27;        { Number of functions }
  NMAX  = 10000000;  { Number of evaluations for each function }

const
  FuncName : array[1..NFUNC] of String[7] =
  ('Ln     ',
   'Log10  ',
   'Log2   ',
   'Exp    ',
   'Exp10  ',
   'Exp2   ',
   'Power  ',
   'Sin    ',
   'Cos    ',
   'Tan    ',
   'ArcSin ',
   'ArcCos ',
   'ArcTan ',
   'ArcTan2',
   'Sinh   ',
   'Cosh   ',
   'Tanh   ',
   'ArcSinh',
   'ArcCosh',
   'ArcTanh',
   'Gamma  ',
   'IGamma ',
   'Beta   ',
   'IBeta  ',
   'Erf    ',
   'Erfc   ',
   'Random ');

var
  F : Text;     { Output file }
  N : Integer;  { Function index }
  T : Float;    { Time }

function TimeToEval(N : Integer) : Float;
{ Returns time to evaluate NMAX functions }
var
  I      : Integer;
  T1, T2 : MicroSecondTimeType;
  Y      : Float;
begin
  T1 := GetMicroSecondTime;
  case N of
    1 : for I := 1 to NMAX do
          Y := Ln(Random);
    2 : for I := 1 to NMAX do
          Y := Log10(Random);
    3 : for I := 1 to NMAX do
          Y := Log2(Random);
    4 : for I := 1 to NMAX do
          Y := Exp(Random);
    5 : for I := 1 to NMAX do
          Y := Exp10(Random);
    6 : for I := 1 to NMAX do
          Y := Exp2(Random);
    7 : for I := 1 to NMAX do
          Y := Random ** 0.5;
    8 : for I := 1 to NMAX do
          Y := Sin(Random);
    9 : for I := 1 to NMAX do
          Y := Cos(Random);
   10 : for I := 1 to NMAX do
          Y := Tan(Random);
   11 : for I := 1 to NMAX do
          Y := ArcSin(Random);
   12 : for I := 1 to NMAX do
          Y := ArcCos(Random);
   13 : for I := 1 to NMAX do
          Y := ArcTan(Random);
   14 : for I := 1 to NMAX do
          Y := ArcTan2(Random, 0.5);
   15 : for I := 1 to NMAX do
          Y := Sinh(Random);
   16 : for I := 1 to NMAX do
          Y := Cosh(Random);
   17 : for I := 1 to NMAX do
          Y := Tanh(Random);
   18 : for I := 1 to NMAX do
          Y := ArcSinh(Random);
   19 : for I := 1 to NMAX do
          Y := ArcCosh(Random + 1.0);
   20 : for I := 1 to NMAX do
          Y := ArcTanh(Random);
   21 : for I := 1 to NMAX do
          Y := Gamma(Random);
   22 : for I := 1 to NMAX do
          Y := IGamma(Random, 0.5);
   23 : for I := 1 to NMAX do
          Y := Beta(Random, 0.5);
   24 : for I := 1 to NMAX do
          Y := IBeta(Random, 0.5, 0.5);
   25 : for I := 1 to NMAX do
          Y := Erf(Random);
   26 : for I := 1 to NMAX do
          Y := Erfc(Random);
   27 : for I := 1 to NMAX do
          Y := Random;
  end;
  T2 := GetMicroSecondTime;
  TimeToEval := (T2 - T1) * 1.0E-6;
end;

begin
  { Open output file }
  Assign(F, 'speed.out');
  Rewrite(F);

  Writeln;
  Writeln('Time in seconds to evaluate ', NMAX, ' functions');
  Writeln(F, 'Time in seconds to evaluate ', NMAX, ' functions');
  Writeln;
  Writeln(F);

  for N := 1 to NFUNC do
    begin
      T := TimeToEval(N);
      Writeln(FuncName[N], T:6:2);
      Writeln(F, FuncName[N], T:8:2);
    end;

  Close(F);
end.
