{ ******************************************************************
  This program computes the Cholesky factorization of a symmetric
  positive definite matrix.

  The file matrix4.inc is an example data file with N = 3
  ****************************************************************** }

program cholesk;

uses
  utypes, ucholesk, ustrings, matutils;

{$i matrix4.inc}

var
  L       : array[1..N, 1..N] of Float;
  I, J, K : Integer;
  S       : Float;

begin
  SetFormat(12, 6, False, True);
  WriteMatrix('Original matrix', A);

  { Perform Cholesky factorization }

  Cholesky(A, L);

  if MathErr = MatNotPD then
    begin
      Writeln('Matrix not positive definite');
      Exit;
    end;

  WriteMatrix('Cholesky factor (L)', L);

  { Check that L * L' = A }

  for I := 1 to N do
    for J := 1 to I do
      begin
        S := 0.0;
        for K := 1 to J do
          S := S + L[I,K] * L[J,K];
        A[I,J] := S;
        if I <> J then
          A[J,I] := S;
      end;

  WriteMatrix('Product L * L''', A);
end.
