{ **********************************************************************
  This program computes the eigenvalues and eigenvectors of a
  general square matrix.

  The file matrix1.inc is an example data file with N = 4.
  ********************************************************************** }

program eigenvec;

uses
  utypes, ueigvec, ustrings, matutils;

{$define Matrix array[Lb1..Ub1 : Integer; Lb2..Ub2 : Integer] of Float}

{$define CompVector array[LbC..UbC : Integer] of Complex}

procedure WriteEigenValue(var Lambda : CompVector; I : Integer);
{ ----------------------------------------------------------------------
  Writes the I-th eigenvalue
  ---------------------------------------------------------------------- }
begin
  WriteLn; WriteLn('Eigenvalue'); WriteLn;
  if Im(Lambda[I]) = 0.0 then
    WriteLn(FloatStr(Re(Lambda[I])))
  else
    WriteLn(CompStr(Lambda[I]));
end;

procedure WriteEigenVector(var V      : Matrix;
                           var Lambda : CompVector;
                               I      : Integer);
{ ----------------------------------------------------------------------
  Writes the I-th eigenvector
  ---------------------------------------------------------------------- }
var
  K : Integer;
begin
  WriteLn; WriteLn('Eigenvector'); WriteLn;
  if Im(Lambda[I]) = 0.0 then
    { Eigenvector is in column I of V }
    for K := Lb1 to Ub1 do
      Writeln(FloatStr(V[K,I]))
  else if Im(Lambda[I]) > 0.0 then
    { Real and imag. parts of eigenvector are in columns I and (I+1) }
    for K := Lb1 to Ub1 do
      Writeln(CompStr(Cmplx(V[K,I], V[K,I+1])))
  else
    { Conjugate of eigenvector is in columns (I-1) and I }
    for K := Lb1 to Ub1 do
      Writeln(CompStr(Cmplx(V[K,I-1], - V[K,I])));
end;

{$i matrix1.inc}

var
  Lambda  : array[1..N] of Complex;      { Eigenvalues }
  V       : array[1..N, 1..N] of Float;  { Eigenvectors }
  I       : Integer;                     { Loop variable }
  ErrCode : Integer;                     { Error code }

begin
  SetFormat(12, 6, False, True);
  WriteMatrix('Original matrix', A);

  { Compute eigenvalues and eigenvectors (A is destroyed) }
  EigenVect(A, Lambda, V);

  { Display results }
  ErrCode := MathErr;
  if ErrCode = 0 then
    for I := 1 to N do
      begin
        WriteEigenValue(Lambda, I);
        WriteEigenVector(V, Lambda, I);
        WriteLn; Write('Press <Enter> to continue...');
        ReadLn;
      end
  else
    begin
      WriteLn('Unable to find eigenvalues Lambda[1] to Lambda[', -ErrCode, ']');
      WriteLn('Eigenvalues Lambda[', 1 - ErrCode, '] to Lambda[', N, ']:');
      for I := 1 - ErrCode to N do
        WriteEigenValue(Lambda, I);
    end;
end.

