{ ******************************************************************
  Correlation and principal component analysis
  ******************************************************************
  Example taken from:
  P. DAGNELIE, Analyse statistique a plusieurs variables,
  Presses Agronomiques de Gembloux, Belgique, 1982
  ****************************************************************** }

program testpca;

uses
  utypes, upca, ustrings, matutils;

const
  Nobs = 11;  { Number of observations }
  Nvar = 4;   { Number of variables }

{ Data }
var X : array[1..Nobs, 1..Nvar] of Float =
(( 87.9, 19.6,   1  , 1661),
 ( 89.9, 15.2,  90.1,  968),
 (153  , 19.7,  56.6, 1353),
 (132.1, 17  ,  91  , 1293),
 ( 88.8, 18.3,  93.7, 1153),
 (220.9, 17.8, 106.9, 1286),
 (117.7, 17.8,  65.5, 1104),
 (109  , 18.3,  41.8, 1574),
 (156.1, 17.8,  57.4, 1222),
 (181.5, 16.8, 140.6,  902),
 (181.4, 17  ,  74.3, 1150));

var
  M      : array[1..Nvar] of Float;           { Mean vector }
  V      : array[1..Nvar, 1..Nvar] of Float;  { Var-cov. matrix }
  R      : array[1..Nvar, 1..Nvar] of Float;  { Correlation matrix }
  S      : array[1..Nvar] of Float;           { Standard deviations }
  Lambda : array[1..Nvar] of Float;           { Eigenvalues of R }
  C      : array[1..Nvar, 1..Nvar] of Float;  { Eigenvectors of R }
  Rc     : array[1..Nvar, 1..Nvar] of Float;  { Correl. factors/var. }
  Z      : array[1..Nobs, 1..Nvar] of Float;  { Scaled variables }
  F      : array[1..Nobs, 1..Nvar] of Float;  { Principal factors }
  I, J   : Integer;                           { Loop variables }

begin
  { Compute mean vector, var-cov. matrix and correlation matrix }
  VecMean(X, M);
  MatVarCov(X, M, V);
  MatCorrel(V, R);

  { Display results }
  SetFormat(12, 4, False, True);
  WriteVector('Mean vector', M);
  WriteMatrix('Variance-covariance matrix', V);
  WriteMatrix('Correlation matrix', R);

  { Compute standard deviations }
  VecSD(X, M, S);

  { Scale variables }
  ScaleVar(X, M, S, Z);

  { Perform principal component analysis
    The original matrix R is destroyed }
  PCA(R, Lambda, C, Rc);

  if MathErr = MatNonConv then
    begin
      Writeln('Non-convergence of eigenvalue computation');
      Exit;
    end;

  { Compute principal factors }
  PrinFac(Z, C, F);

  { Display results }
  Writeln;
  WriteVector('Eigenvalues of correlation matrix', Lambda);
  WriteMatrix('Eigenvectors (columns) of correlation matrix', C);
  WriteMatrix('Correlations between factors (columns) and variables (lines)', Rc);
  WriteMatrix('Principal factors', F);
end.

