{ ******************************************************************
  This program simulates a bi-lognormal distribution. The results 
  are stored in an output file (one random vector by line)
  ****************************************************************** }

program ranmull;

uses
  utypes, ucholesk, uranmult;

const
  N    = 2;       { Size of distribution }
  NSIM = 100;     { Number of simulations }
  R    = 0.5672;  { Correlation coefficient }

var  
  M    : array[1..N] of Float = (17.4178, 5.3173);  { Means }
  S    : array[1..N] of Float = ( 6.1259, 2.5158);  { S.D. }

  M0   : array[1..N] of Float;        { Transformed means }
  X, Z : array[1..N] of Float;        { Random vectors }
  V    : array[1..N, 1..N] of Float;  { Variance-covariance matrix }
  V0   : array[1..N, 1..N] of Float;  { Transformed var. cov. matrix }
  L    : array[1..N, 1..N] of Float;  { Cholesky factor of V0 }
  
  F    : Text;     { Output file }
  I, J : Integer;  { Loop variables }

begin
  for I := 1 to N do
    V[I,I] := Sqr(S[I]);      { Variances }
  
  V[1,2] := R * S[1] * S[2];  { Covariance }

  { Define auxiliary normal distribution N(M0, V0) }
  for I := 1 to N do
    begin
      for J := 1 to N do
        V0[I,J] := Ln(V[I,J] / (M[I] * M[J]) + 1.0);
      M0[I] := Ln(M[I]) - 0.5 * V0[I,I];
    end;

  { Perform Cholesky decomposition of variance-covariance matrix }
  Cholesky(V0, L);
  if MathErr = MatNotPD then
    begin
      WriteLn('Variance-covariance matrix is not positive definite.');
      Exit;
    end;

  Assign(F, 'ranmull.out');
  Rewrite(F);
  
  for I := 1 to NSIM do
    begin
      { Pick random vector from auxiliary normal distribution }
      RanMult(L, M0, Z);

      { Convert to lognormal }
      for J := 1 to N do
        X[J] := Exp(Z[J]);

      { Output result to file }
      for J := 1 to N do
        Write(F, X[J]:12:6);
      Writeln(F);
    end;
  Close(F);
end.
