{ ******************************************************************
  Bartlett's test (comparison of several variances)
  ****************************************************************** }

unit ubartlet;

interface

uses
  utypes;

{$define Vector array[Lb..Ub : Integer] of Float}

{$define IntVector array[LbI..UbI : Integer] of Integer}

procedure Bartlett(var N    : IntVector;
                   var S    : Vector;
                   var Khi2 : Float;
                   var DoF  : Integer);
{ ------------------------------------------------------------------
  Input parameters : N    = samples sizes
                     S    = samples SD's (computed with StDev)
  Output parameters: Khi2 = Bartlett's khi-2
                     DoF  = degrees of freedom
  ------------------------------------------------------------------ }

implementation

procedure Bartlett(var N    : IntVector;
                   var S    : Vector;
                   var Khi2 : Float;
                   var DoF  : Integer);
var
  I, Ns, Nt, N1, DoF_r        : Integer;
  SSr, Vr, Vi, SumLog, SumInv : Float;

begin
  if (Lb <> 1) or (Ub < 2) or (LbI <> Lb) or (UbI <> Ub) then
    begin
      SetErrCode(MatErrDim);
      Exit
    end;

  Ns := Ub;
  Nt := 0;
  for I := 1 to Ns do
    Nt := Nt + N[I];

  if Nt <= Ns then
    begin
      SetErrCode(MatErrDim);
      Exit;
    end;

  SetErrCode(MatOk);

  SSr := 0.0;
  SumLog := 0.0;
  SumInv := 0.0;

  for I := 1 to Ns do
    begin
      N1 := N[I] - 1;
      Vi := Sqr(S[I]);
      SSr := SSr + N1 * Vi;
      SumLog := SumLog + N1 * Ln(Vi);
      SumInv := SumInv + 1 / N1;
    end;

  DoF := Ns - 1;
  DoF_r := Nt - Ns;
  Vr := SSr / DoF_r;
  Khi2 := (DoF_r * Ln(Vr) - SumLog) /
          (1.0 + (SumInv - 1.0 / DoF_r) / (3.0 * DoF));
end;

end.
