{ ******************************************************************
  Trapezoidal integration
  ****************************************************************** }

unit utrapint;

interface

uses
  utypes;

{$define Vector array[Lb..Ub : Integer] of Float}

function TrapInt(var X, Y : Vector) : Float;
{ Integration by trapezoidal rule }

procedure ConvTrap(Func1, Func2 : TFunc; var T, Y : Vector);
{ ------------------------------------------------------------------
  Computes the convolution product of 2 functions Func1 and Func2
  by the trapezoidal rule over an array T[0..N] of equally spaced
  abscissas, with T[0] = 0. The result is returned in Y[0..N]
  ------------------------------------------------------------------ }

implementation

function TrapInt(var X, Y : Vector) : Float;
var
  Sum  : Float;
  I, J : Integer;
begin
  Sum := 0.0;
  for I := Lb to Pred(Ub) do
    begin
      J := Succ(I);
      Sum := Sum + 0.5 * (X[J] - X[I]) * (Y[J] + Y[I]);
    end;
  TrapInt := Sum;
end;

procedure ConvTrap(Func1, Func2 : TFunc; var T, Y : Vector);
{ ------------------------------------------------------------------
  Computes the convolution product of 2 functions Func1 and Func2
  by the trapezoidal rule over an array T[0..N] of equally spaced
  abscissas, with T[0] = 0. The result is returned in Y[0..N]
  ------------------------------------------------------------------ }
var
  F2       : array[0..Ub] of Float;
  F10, Sum : Float;
  I, J     : Integer;
begin
  F10 := Func1(0.0);

  for I := 0 to Ub do
    F2[I] := Func2(T[I]);

  for I := 1 to Ub do
    begin
      Sum := 0.5 * (Func1(T[I]) * F2[0] + F10 * F2[I]);
      for J := 1 to I - 1 do
        Sum := Sum + Func1(T[I] - T[J]) * F2[J];
      Y[I] := Sum * T[1];
    end;
end;

end.
